@@ -29,15 +29,14 @@ tool for planned events as well as diagnosis during break fix.
 import logging
 import optparse
 import os
-import re
 import sys
 import threading
 
 try:
     import netmunge
-except:
+except ImportError:
     netmunge = None
-    
+
 import notch.client
 
 import cmdline
@@ -52,7 +51,7 @@ class MisterCLI(cmdline.CLI):
     PROMPT = '%s [t: 0] > ' % PREFIX
 
     def __init__(self, notch, completekey='tab', stdin=None, stdout=None,
-                 menu=None, targets=None):
+                 targets=None):
         menu = {r'exit': 'do_exit',
                 r'quit': 'do_exit',
                 r'help': 'do_help',
@@ -66,7 +65,8 @@ class MisterCLI(cmdline.CLI):
                 }
         super(MisterCLI, self).__init__(menu, completekey=completekey,
                                         stdin=stdin, stdout=stdout)
-        
+        # If False, we won't auto-fallback.
+        self.from_cmd_loop = True
         self.notch = notch
         self.targets = []
 
@@ -96,7 +96,7 @@ class MisterCLI(cmdline.CLI):
                     return True
         return False
 
-    def default(self, line):
+    def default(self, line, action=None):
         self.stdout.write('Error: Unknown command: %s. Try "help".\n' % line)
 
     def do_command(self, line):
@@ -133,7 +133,7 @@ class MisterCLI(cmdline.CLI):
           > output csv     [note: requires python netmunge module]
 
           > output text    [note: default]
-          
+
           > output ...
 
         """
@@ -147,7 +147,8 @@ class MisterCLI(cmdline.CLI):
             mode = line.split()[1]
             if self._output_mode_is_ok(mode):
                 self.output_mode = mode
-                self.stdout.write('Changed to output mode: %s\n' % mode)
+                if self.from_cmd_loop:
+                    self.stdout.write('Changed to output mode: %s\n' % mode)
 
     def do_counters(self, _):
         """Displays the Notch request counters.
@@ -156,7 +157,7 @@ class MisterCLI(cmdline.CLI):
 
         > counters
         Notch Transport Counters
-        [Requests]  total: 97        ok: 97        error: 0        
+        [Requests]  total: 97        ok: 97        error: 0
         [Responses] total: 97        ok: 85        error: 12        data: 1.2 MB
 
         ------------------------------------------------------------------------
@@ -193,16 +194,12 @@ class MisterCLI(cmdline.CLI):
         if not arg or len(arg) == 1:
             return
         else:
-            try:
-                targets = self._get_targets(line, only_regexp=True)
-            except NoDevicesMatching, e:
-                self.stdout.write('Targets could not be retrieved.')
+            targets = self._get_targets(line, only_regexp=True)
+            if targets:
+                self.stdout.write('Matching device names (%d): %s\n'
+                                  % (len(targets), str(', '.join(targets))))
             else:
-                if targets:
-                    self.stdout.write('Matching device names (%d): %s\n'
-                                      % (len(targets), str(', '.join(targets))))
-                else:
-                    self.stdout.write('No targets matched your query.\n')
+                self.stdout.write('No targets matched your query.\n')
 
     def do_notallowed(self, line):
         """This command is disallowed."""
@@ -253,7 +250,7 @@ class MisterCLI(cmdline.CLI):
         """Displays or sets the timeout, in seconds.
 
         Issuing the 'timeout' command will display the current timeout.
-        
+
         To set the timeout, supply an integer or floating point value
         argument.
 
@@ -266,7 +263,7 @@ class MisterCLI(cmdline.CLI):
           > timeout 0.5
           Error: 1 second is the minimum timeout.
           Timeout is 5.0 seconds.
-        
+
           > timeout
           Timeout is 5.0 seconds.
         """
@@ -299,14 +296,13 @@ class MisterCLI(cmdline.CLI):
                 if not only_regexp:
                     result.append(t)
         return result
-                
+
     def _parse_targets(self, line):
         """Parses the targets argument."""
         args = line.split()
         if len(args) < 2:
             return []
         else:
-            targets = []
             target_spec = args[1:]
             target_spec = [t.split(',') for t in target_spec]
             specs = []
@@ -340,12 +336,11 @@ class MisterCLI(cmdline.CLI):
             method_args = {'device_name': target,
                            'command': command}
             kwargs = {'output_method': output_method}
-            timeout = self.timeout
             r = notch.client.Request('command',
                                      arguments=method_args,
                                      callback=self._notch_callback,
                                      callback_kwargs=kwargs,
-                                     timeout_s=timeout)
+                                     timeout_s=self.timeout)
             reqs.append(r)
         logging.debug('Executing %d requests.', len(reqs))
         gts = self.notch.exec_requests(reqs)
@@ -354,9 +349,9 @@ class MisterCLI(cmdline.CLI):
                 for gt in gts:
                     logging.debug('Waiting for %r', gt)
                     gt.wait()
-            except notch.client.TimeoutError, e:
-                self.stdout.write('%s: request timed out (%.1f s).\n' %
-                                  (target, timeout))
+            except notch.client.TimeoutError:
+                self.stdout.write('Request timed out (%.1f s).\n' %
+                                  self.timeout)
         else:
             self.notch.wait_all()
 
@@ -370,7 +365,8 @@ class MisterCLI(cmdline.CLI):
                                str(request.error)))
 
     def _notch_callback(self, request, *args, **kwargs):
-        output_method = kwargs.get('output_method', None)
+        _ = args
+        output_method = kwargs.get('output_method')
         request.finish()
         if output_method is not None and hasattr(
             self, '_output_' + output_method):
@@ -395,7 +391,7 @@ class MisterCLI(cmdline.CLI):
             device_type = device.get('device_type')
         else:
             device_type = 'UNKNOWN_DEVICE'
-       
+
         if command is None:
             logging.warn('Not a command request. Not sure how to proceed.')
             return
@@ -409,14 +405,20 @@ class MisterCLI(cmdline.CLI):
                         fields = ','.join(r)
                         self.stdout.write('%s,%s\n' % (device_name,fields))
                 else:
-                    # Output parsing probably failed, so just report what we got.
-                    results = request.result
-                    self.stdout.write('%s:\n%s\n' % (device_name, results))
+                    # Only report raw results if we're falling back.
+                    if self.from_cmd_loop:
+                        results = request.result
+                        self.stdout.write('%s:\n%s\n' % (device_name, results))
 
             except ValueError:
-                # No parser available for this command; use regular output.
-                results = request.result
-                self.stdout.write('%s:\n%s\n' % (device_name, results))
+                # If we ran this from the loop (i.e., interactive Mr. CLI),
+                # we'll automatically fall-back to raw text mode. If not,
+                # don't display anything (as the data is likely going to be
+                # parsed and we cannot make guarantees about complete datasets).
+                if self.from_cmd_loop:
+                    # No parser available for this command; use regular output.
+                    results = request.result
+                    self.stdout.write('%s:\n%s\n' % (device_name, results))
         else:
             self._print_error(request)
 
@@ -431,7 +433,7 @@ class MisterCLI(cmdline.CLI):
             self._print_error(request)
         else:
             self.stdout.write('%s: Incomplete response from Notch Agent.\n' %
-                              (device_name))
+                              device_name)
 
     def _output_text(self, request):
         device_name = request.arguments.get('device_name')
@@ -442,7 +444,7 @@ class MisterCLI(cmdline.CLI):
             self._print_error(request)
         else:
             self.stdout.write('%s: Incomplete response from Notch Agent.\n' %
-                              (device_name))
+                              device_name)
 
     def interrupted(self):
         if self.notch.num_requests_running or self.notch.num_requests_waiting:
@@ -464,11 +466,13 @@ def get_option_parser():
             'Examples:',
             '    $ %s localhost:8080,localhost:8081' % prog,
             '',
+            '    $ %s  # Use .notchrc or /etc/notchrc for agent hosts' % prog,
+            '',
             '    $ NOTCH_AGENTS="localhost:8080,server.example.com:8080" %s'
             % prog,
             '',
             '    $ export NOTCH_AGENTS="localhost:8080"',
-            '    $ %s -t cr1.mel -c "show arp"' % prog,
+            '    $ %s -t cr1.mel -o csv -c "show arp"' % prog,
             ])
 
     parser.add_option('-t', '--target', dest='targets', action='append',
@@ -476,20 +480,23 @@ def get_option_parser():
                       help='Adds a single target device')
     parser.add_option('-c', '--cmd', dest='cmd', default=None,
                       help='The command to execute on each target')
+    modes = ['text']
+    if netmunge:
+        modes.append('csv')
+    parser.add_option('-o', '--output', dest='mode', default=None,
+                      help='Use output mode (%s)'
+                      % ', '.join(modes))
     return parser
 
 
 WELCOME_MSG = 'Welcome to Mr. CLI.  Type \'help\' if you need it.'
 
-def main():
-    option_parser = get_option_parser()
-    options, args = option_parser.parse_args()
 
+def _get_agents(args):
     # Attempt to gather agent addresses from the environment.
     agents = os.getenv('NOTCH_AGENTS')
     if not args and not agents:
-        print option_parser.format_help()
-        raise SystemExit(1)
+        return None
 
     # Figure out the agent addresses from the commandline.
     if not agents:
@@ -497,19 +504,30 @@ def main():
         for i, a in enumerate(agents):
             agents[i] = a.lstrip()
 
+    return agents
+
+
+def main():
+    option_parser = get_option_parser()
+    options, args = option_parser.parse_args()
+
+    agents = _get_agents(args)
     # Start the Notch client and CLI
     try:
         nc = notch.client.Connection(agents)
         cli = MisterCLI(nc, targets=options.targets)
 
         if options.cmd:
+            cli.from_cmd_loop = False
+            if options.mode != 'text':
+                cli.do_output('output %s' % options.mode)
             cli.do_command('cmd %s' % options.cmd)
         else:
             print WELCOME_MSG
             cli.cmdloop()
             print '\nBye.'
     except notch.client.NoAgentsError, e:
-        print 'No valid agents were found.'
+        print str(e)
         print
         print option_parser.get_usage()
         raise SystemExit(1)
